
#    Moving window De-convolution parameter setup for grt4p module

set frame grt4pmwd
set title "GRT 4P Module MWD Parameter Setup Interface"
set icontitle grt4p


set moduletype "GRT 4P"
set modulename  grt4p
set debug 0

set maxmodule 8
set modulelist {}
set module 1
set channellist {A B C D}
set allchannels 0
set allmodules 0

set mwdchannel A

# Input parameters

set clock_frequency      80.0
set decimation_factor    4
set shaping_time         6.0
set trapezoid_width      4.4
set peak_separation      8.5
set decay_time_constant  50.0
set peak_sample_time     7.2
set baseline_update_time 2.0 
set baseline_average     4
set mode                 0

#Calculated Variables

set clock_width          [expr 1000 / $clock_frequency]

# Download parameters

loadsource common.vme

proc make-mwd-frame {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time baseline_update_time
    global baseline_average mode
    global debug maxmodule modulelist module mwdchannel allmodules allchannels channellist

    for {set i 0} {$i < 11} {incr i} { global MWDp$i; set MWDp$i "" }

  :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F \
      :icon-name "[self icon-title]"

   :panel P1 :layout L :padx 10 :pady 5
    :layout V; :row-gap 10; :col-gap 10
    
    :col-gap 20
    set xpos 150
   :text CLOCK_FREQUENCY      :label clock_frequency  :w 6 :valuex $xpos
    :text DECIMATION_FACTOR    :label decimation_factor  :w 6 :valuex $xpos
    :text SHAPING_TIME         :label shaping_time     :w 6 :valuex $xpos
    :text TRAPEZOID_WIDTH      :label trapezoid_width   :w 6 :valuex $xpos
    :text PEAK_SEPARATION      :label peak_separation  :w 6 :valuex $xpos
    :text DECAY_TIME_CONSTANT  :label decay_time_constant  :w 6 :valuex $xpos
    :text PEAK_SAMPLE_TIME     :label peak_sample_time     :w 6 :valuex $xpos
    :text BASELINE_UPDATE_TIME :label baseline_update_time   :w 6 :valuex $xpos
    :text BASELINE_AVERAGE     :label baseline_average   :w 6 :valuex $xpos
    :text MODE                 :label mode   :w 6 :valuex $xpos
    
    :panel P2 :layout R :padx 10 :pady 5
    :layout V; :row-gap 10; :col-gap 10

    set xpos 100
    for {set i 0} {$i < 11} {incr i} {
	:text MWDP$i :label "MWDP$i" :value [set MWDp$i] :w 10 :valuex $xpos
    }

    :panel P3 :layout B :padx 10 :pady 5
    :layout V; :row-gap 10; :col-gap 10

    set modulelist [inform grt4p get-modulelist]

    :number MODULE-NUM  :label "Module number:" :min 1 :max $maxmodule :w 5 :value $module
    :menubutton MODULE-SEL :label "select:" :strings $modulelist
    :vskip 2
    :message CHANNEL-NUM :label "Channel $mwdchannel" :bold T
    :menubutton CHANNEL-SEL :label "select:" :strings $channellist
    :vskip 10
    :checkbox ALLCHANNELS :label "Act on all channels" :value $allchannels
    :checkbox ALLMODULES  :label "Act on all modules"  :value $allmodules
    :vskip 50
    :button APPLY   :label "Apply parameters"
    :button READREGS  :label "Read parameters"

 :show T
}

proc modify-mwd-parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time baseline_update_time
    global baseline_average mode
    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    :modify CLOCK_FREQUENCY      :value $clock_frequency
    :modify DECIMATION_FACTOR    :value $decimation_factor
    :modify SHAPING_TIME         :value $shaping_time
    :modify TRAPEZOID_WIDTH      :value $trapezoid_width
    :modify PEAK_SEPARATION      :value $peak_separation
    :modify DECAY_TIME_CONSTANT  :value $decay_time_constant
    :modify PEAK_SAMPLE_TIME     :value $peak_sample_time
    :modify BASELINE_UPDATE_TIME :value $baseline_update_time
    :modify BASELINE_AVERAGE     :value $baseline_average
    :modify MODE                 :value $mode

    for {set i 0} {$i < 11} {incr i} {
	:modify MWDP$i :value [set MWDp$i] }
}



proc click-clock_frequency  {v} {
global clock_frequency clock_width

set clock_frequency $v

redisplay
}

proc click-decimation_factor  {v} {
global decimation_factor

set decimation_factor $v

redisplay
}

proc click-shaping_time  {v} {
global shaping_time

set shaping_time $v

redisplay
}

proc click-trapezoid_width   {v} {
global trapezoid_width

set trapezoid_width $v

redisplay
}

proc click-peak_separation  {v} {
global peak_separation

set peak_separation $v

redisplay
}

proc click-decay_time_constant  {v} {
global decay_time_constant

set decay_time_constant $v
 
redisplay
}

proc click-peak_sample_time   {v} {
global peak_sample_time

set peak_sample_time $v

redisplay
}

proc click-baseline_update_time   {v} {
global baseline_update_time

set baseline_update_time $v

redisplay
}

proc click-baseline_average {v} {
global baseline_average

set baseline_average $v

redisplay
}

proc click-mode {v} {
global mode

set mode $v

redisplay
}

proc update-mwd-parameters {} {
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time baseline_update_time
    global baseline_average mode
    global clock_width dt Mbydt
    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    set clock_width [expr 1000000000 / ( [set clock_frequency] * 1000000 ) ]
    set ddt [expr $clock_width * 0.001 * $decimation_factor ]
    set dt [expr $clock_width * 0.001 ]
    set Mbydt [expr $shaping_time / $ddt]

    set MWDp0 [expr round(256 - $Mbydt +1)]
    set MWDp1 [expr 256 - round($trapezoid_width / $ddt ) -1]
    set MWDp2 [expr int($peak_sample_time / $dt)]
    set MWDp3 [expr int( $peak_separation / $dt ) ]
    set MWDp4 $baseline_average
    set MWDp5 [expr round($baseline_update_time / $ddt)]
    set MWDp6  16384

    set value [expr round(16777216 * (1 - exp([expr - $shaping_time / $decay_time_constant])) / $Mbydt)]
    set lower [set value]
    if { $lower > 65535} {set lower [expr $lower -65535]} 
    set MWDp7 [set lower]
    set MWDp8 [expr int($value / 65536)]

    set MWDp9 [expr 32 - $decimation_factor +1]
    set MWDp10 $mode

}

proc apply-mwd-parameters {} {
    global allmodules allchannels modulename module mwdchannel
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time baseline_update_time
    global baseline_average mode
    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    if {$allchannels} {set ch ? } {set ch $mwdchannel}
    set m $modulename#$module
    if {$allmodules} {set m [string replace $m end end ?]}

    write-vme-registers $m.Ch$ch.MWDPclkfreq $clock_frequency
    write-vme-registers $m.Ch$ch.MWDPdecfact $decimation_factor
    write-vme-registers $m.Ch$ch.MWDPshptime $shaping_time
    write-vme-registers $m.Ch$ch.MWDPtrapwid $trapezoid_width
    write-vme-registers $m.Ch$ch.MWDPpeaksep $peak_separation
    write-vme-registers $m.Ch$ch.MWDPdecaytc $decay_time_constant
    write-vme-registers $m.Ch$ch.MWDPpeakstm $peak_sample_time
    write-vme-registers $m.Ch$ch.MWDPbaseutm $baseline_update_time
    write-vme-registers $m.Ch$ch.MWDPbaseave $baseline_average
    write-vme-registers $m.Ch$ch.MWDPmode $mode

    for {set i 0} {$i < 11} {incr i} {
	write-vme-registers $m.Ch$ch.MWD$i  [set MWDp$i]
    }

#    perform a soft reset to cause MWD parameters to be loaded

    write-vme-registers $m.Ch$ch.ADC_Ctrl.b0 1
    write-vme-registers $m.Ch$ch.ADC_Ctrl.b0 0

}

proc read-mwdp-parameters {} {
    global modulename module mwdchannel
    global clock_frequency decimation_factor shaping_time  trapezoid_width
    global peak_separation decay_time_constant peak_sample_time baseline_update_time
    global baseline_average mode

    set m $modulename#$module
    set clock_frequency [read-vme-register $m.Ch$mwdchannel.MWDPclkfreq]
    set decimation_factor [read-vme-register $m.Ch$mwdchannel.MWDPdecfact]
    set shaping_time [read-vme-register $m.Ch$mwdchannel.MWDPshptime]
    set trapezoid_width [read-vme-register $m.Ch$mwdchannel.MWDPtrapwid]
    set peak_separation [read-vme-register $m.Ch$mwdchannel.MWDPpeaksep]
    set decay_time_constant [read-vme-register $m.Ch$mwdchannel.MWDPdecaytc]
    set peak_sample_time [read-vme-register $m.Ch$mwdchannel.MWDPpeakstm]
    set baseline_update_time [read-vme-register $m.Ch$mwdchannel.MWDPbaseutm]
    set baseline_average [read-vme-register $m.Ch$mwdchannel.MWDPbaseave]
    set mode [read-vme-register $m.Ch$mwdchannel.MWDPmode]

}

set MWD_MASKS { 
    0x00ff 0x00ff 0xffff 0xffff 0x00ff 0x00ff 0xffff 0xffff 0x00ff 0x001f 0x0003 }


proc read-mwd-parameters {} {
    global modulename module mwdchannel MWD_MASKS
    for {set i 0} {$i < 11} {incr i} { global MWDp$i }

    set m $modulename#$module
    for {set i 0} {$i < 11} {incr i} {
	set MWDp$i [expr [read-vme-register $m.Ch$mwdchannel.MWD$i] & [lindex $MWD_MASKS $i]]
    }


    modify-mwd-parameters
}

proc redisplay {} {
    global mwdchannel
    :modify CHANNEL-NUM :label "Channel $mwdchannel" :bold T
    update-mwd-parameters
    modify-mwd-parameters
}

proc click-select-module {v} {
    global modulename module
    set module $v
    redisplay
}
  
proc click-select-channel {v} {
    global mwdchannel
    set mwdchannel $v
    redisplay
}
  
proc click arg {
   global debug allmodules allchannels
   if {$debug} then {puts stderr "click with argument \{$arg\}"}
#   insert-debug "click with argument \{$arg\}"
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch $w {

       MODULE-SEL      -
       MODULE-NUM      {click-select-module $v}
       ALLMODULES   {set allmodules $v}
       CHANNEL-SEL  {click-select-channel $v}
       ALLCHANNELS  {set allchannels $v}

       CLOCK_FREQUENCY      {click-clock_frequency $v}
       DECIMATION_FACTOR    {click-decimation_factor $v}
       SHAPING_TIME         {click-shaping_time $v}
       TRAPEZOID_WIDTH      {click-trapezoid_width $v}
       PEAK_SEPARATION      {click-peak_separation $v}
       DECAY_TIME_CONSTANT  {click-decay_time_constant $v}
       PEAK_SAMPLE_TIME     {click-peak_sample_time $v}
       BASELINE_UPDATE_TIME {click-baseline_update_time $v}
       BASELINE_AVERAGE     {click-baseline_average $v}
       MODE                 {click-mode $v}

       APPLY                {apply-mwd-parameters}
       READREGS             {read-mwd-parameters}

       default   {puts stdout $arg}
   }
}

proc start {args} {
   global debug modulename module mwdchannel modulelist
    if {$debug} {puts "MWD start with $args "}
    set modulename [first $args]
    set module [second $args]
    set mwdchannel [third $args]

    make-mwd-frame
    read-mwdp-parameters
    update-mwd-parameters
    modify-mwd-parameters

}









