#    most of this task is now taken from the standard application

set VERSION "GRT4P"

#    put some default environment variables before loading common 

if {[info exists env(VMEServer)]} then {set resource $env(VMEServer)} else {set resource VME}
set VMEserver $resource

if {[info exists env(RDOUTserver)]} then {set resource $env(RDOUTserver)} else {set resource VME}
set RDOUTserver $resource

set RDOUTserverList $RDOUTserver
set SpectrumServerList {}
set SpectrumServer [first $SpectrumServerList]

set DAProgram GRT4PExecV6

loadsource  sys.common

set frametitle "MIDAS Data Acquisition Experiment Control (GRT4P)"


set VMEPresent 1
set VMEEnable 1

set TSPresent 1
set TSEnable 0
set txEnable 0

set histEnable 1
set HistMode 1

set TestBuffer ""

set Supported_Modules [list GRT S9418 V785 V775 VME]

set S9418_Base 64
set Options(s9418_GroupBase) $S9418_Base

set V785_Base 80
set V775_Base 96
set Options(v785_GroupBase) $V785_Base
set Options(v775_GroupBase) $V775_Base

update_Options

lappend setuplist [list "Show histogram name translations" "Show_Xlate"]
lappend setuplist [list "Edit histogram translation file"  "edit_xlate"]
lappend setuplist [list "VME data source configuration"    "Configure_VME_Data_Sources"]
lappend setuplist [list "2D Histogram Configuration"       "2D_Histogram_Configuration"]
lappend setuplist [list "Create 2D Histograms"             "Create_2D_Histograms"]


lappend actionlist [list "Raw Data Display"  "Dump_Raw_VME_Data"]
lappend actionlist [list "Print Hardware Configuration"  "PrintConfig"]

loadsource sys.2D    ;# for 2D histogrammming feature 


proc create_spectra {} {
   global SpectrumServer HistMode EGreport Options
   global Supported_Modules

   if {$HistMode == 0} {return}

   set-footer "Creating spectra"

   set EGreport_SAVE $EGreport
   set EGreport -1
   
   Make_Xlate $SpectrumServer

   set range $Options(Stat.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Stat' !new)" } m]
   map_spectrum "Stat"

   set range $Options(Rate.channels)
   set z [catch {eg "(spectrum :path '/$SpectrumServer' :range $range :name 'Rate' !new)" } m]

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1 &&  $Options(Histogram.$module) == 1} then {
           create_[set module]_spectra
       }
   }

   global 2Dlist 2Dpresent
   if {[llength $2Dlist] == 0} {
       if {$2Dpresent == 0} {2D_Histogram_Configuration; close-frame "[self frame-name].2dhistograms"}
   }
   Create_2D_Histograms

   set EGreport $EGreport_SAVE

   clear-footer
}

proc define_VME_addresses {} {
   global Supported_Modules Options

#    load hardware configuration into readout software

     ClearConfig

   foreach module $Supported_Modules {
       if {$Options(Include.$module) == 1} then {define_[set module]_config}
   }

#    then the acquisition control module

     define_sac_config
}

proc Custom_PreSetup {args} {define_VME_addresses}

proc redisplay-vme {} {
       global RegisterServer
       global VMEEnable vmestate EGallow

       if {$VMEEnable == 0} then {return}

       set RegisterServer [inform vmecontrol inquire Role]

       set vl [read-registers xvme260#1.PortA]
       if {$vl != ""} then {
           if {[second [first $vl]] == "0x0000"} {:modify VME :value 1} else {:modify VME :value 0}
           return
       }
       set vl [read-registers sac#1.Enable]
       if {$vl != ""} then {
           :modify VME :value [expr [second [first $vl]] & 1]]
           return
       }
       :modify VME :value -1
}

proc Stop_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       write-registers xvme260#1.PortA 0

       write-registers sac#1.STR 0
}

proc Go_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

    set maxmodule [inform vmecontrol inquire s9418count]
    if {[string length $maxmodule] > 2} {set maxmodule 0}
       
    if {$maxmodule == 0} then {
        write-registers sac#1.Mode 1   ;#   CAEN mode
    } else {
        write-registers sac#1.Mode 0   ;#   Silena 9418 mode (default)
    }

       write-registers sac#1.MON0 2
       write-registers sac#1.STR 9
       write-registers sac#1.ARM 0

       write-registers xvme260#1.PortA -1
}

proc Reset_Electronics {} {
       global RegisterServer
       set RegisterServer [inform vmecontrol inquire Role]

       click-vme 2  ;#  Called during GO - Will cause Soft Reset of all channels to flush FIFOs etc

       write-registers sac#1.EVCLR 0

       write-registers xvme260#1.PortA 0
}

set scantable 0
set usetime 0
set usecount 0
set verbose 0
set timerange 100

proc do_scantable {w v} {
    global scantable
 
    set scantable [expr $v*2]
    execute-command 16 "" "$scantable 0 0 0 0 0" 2
}

proc do_option {w v} {
    global usetime usecount verbose timerange

    switch $w {
        VERBOSE   {set verbose $v}
        USETIME   {set usetime $v}
        USECOUNT  {set usecount $v}
        TIMERANGE {set timerange $v}
    }

    execute-command 17 "" "$verbose $usetime $usecount $timerange" 2
}

proc do_clearstats {} {execute-command 7  "Clear Statistics"}

proc enable_scanning {} {execute-command 16 "" "3 1 2 3 4 5" 2}

proc click-histenable {v} {
    global RDOUThdwre RDOUTEnable histEnable
    if {$RDOUTEnable == 0} then {set  histEnable 0} else {set histEnable $v}
    if {$RDOUTEnable != 0} then {write-register NetVar.DACQ.Chistenable [expr $histEnable * 10]}
    redisplay-histenable
}
